% Computing image gradients
% Page 99-104 in Ma et al (2006)
% Inputs
%    I         Image
%    sigmas    standard deviation for the smoothing
%    sigma     standard deviation for the differentiation
%
% Outputs
%    grad.Ix   Image gradient in the x direction
%    grad.Iy   Image gradient in the y direction

function grad = iGradient(I,sigmas,sigma)
% Note that sice both the smoothing and the gradients are computed as
% convolutions they commute. Hence, it does not matter in which order they
% are performed.

% Smooth the image in order to get rid of noise
x = -floor(5*sigmas/2):1:floor(5*sigmas/2);
g = (1/(sqrt(2*pi)*sigmas))*exp(-(x.^2)/(2*sigmas^2))';         % Gaussian
Is = conv2(double(I),g*g','same');

% Compute the image gradient
x = -floor(5*sigma/2):1:floor(5*sigma/2);
g = (1/(sqrt(2*pi)*sigma))*exp(-(x.^2)/(2*sigma^2))';         % Gaussian
dg = (-x'/(sqrt(2*pi)*sigma^3)).*exp(-(x'.^2)/(2*sigma^2));   % Derivative of Gaussian

dx = dg*g';
[ma,na] = size(Is);
[mb,nb] = size(dx);
grad.Ix = zeros(ma,na);  % Make sure the gradients are of the same size as I, add zeros
grad.Iy = zeros(ma,na);  % Make sure the gradients are of the same size as I, add zeros

d1=(mb-1)/2;
d2=(nb-1)/2;
grad.Ix(d1+1:end-d1,d2+1:end-d2) = conv2(double(Is),dx,'valid');   % OBS VALID!!!
dy = g*dg';
grad.Iy(d1+1:end-d1,d2+1:end-d2) = conv2(double(Is),dy,'valid');   % OBS VALID!!!
