function [x, P] = IMUBarUpdate(x, P, u, y, par)
%IMUBARUPDATE   Updates the state using the information in the input variables and the 
%   measurements. Note that we have choosen to model the accelerations and
%   the angular velocities as input variables and not measurements.
%
%   Input
%      x           State mean
%      P           State covariance
%      u           Input variables (acceleration and angular velocity)
%      y           Measurements (barometer)
%      par         Parameters
%
%   Output
%      x           State mean
%      P           State covariance

  x = x(1:10);        % Only the vehicle states are needed
  T = par.T;

  ua = u(1:3);
  uw = u(4:6);
  
  b = x(1:3);
  bdot = x(4:6);
  q = x(7:10);

  ae = (Rq(q))'*ua + par.ge;
  b_new = b + T*bdot + ((T^2)/2)*ae;
  bdot_new = bdot + T*ae;
  q_new = (eye(4) + (T/2)*Somega(uw))*q;
  q_new = q_new/norm(q_new);                 % Normalize the quaternion
  x = [b_new; bdot_new; q_new];

  Q = blkdiag((T^2)/2*diag(par.Qa), T*diag(par.Qa), (T/2)*diag(par.Qw));
  q0 = q(1);   q1 = q(2);   q2 = q(3);   q3 = q(4);
  u1 = ua(1);  u2 = ua(2);  u3 = ua(3);
  dRydq = 2*[(q0*u1 - q3*u2 + q2*u3)  (q1*u1 + q2*u2 + q3*u3) (-q2*u1 + q1*u2 + q0*u3) (-q3*u1 - q0*u2 + q1*u3);
             (q3*u1 + q0*u2 - q1*u3)  (q2*u1 - q1*u2 - q0*u3) (q1*u1 + q2*u2 + q3*u3)  (q0*u1 - q3*u2 + q2*u3);
             (-q2*u1 + q1*u2 + q0*u3) (q3*u1 + q0*u2 - q1*u3) (q1*u1 + q3*u2 - q2*u3)  (q1*u1 + q2*u2 + q3*u3)];
  F = [eye(3)   T*eye(3) (T^2)/2*dRydq;
       zeros(3) eye(3)   T*dRydq;
       zeros(4,3) zeros(4,3) eye(4) - (T/2)*Somega(uw)];
  P = F*P*F' + Q;

  % Measurement update using the barometer measurement
  C = [0 0 1 zeros(1,7)];
  K = P*C'*inv(C*P*C' + par.Rbar);
  x = x + K*(y - C*x);
  P = P - K*C*P;
end

