function [yfm, yfp, yhatfp, yhatfp_noinit, visible, visInnovation] = dataAssociation(yhatf, I, x, par, patches, status)
%DATAASSOCIATION   Performs data association, i.e., tries to find the already existing 
%   landmarks in the new image.
%   Input
%      yhatf           Predicted feature position in metric scale (normalized coordinates)
%      yfp             Measured feature position in pixels 
%      I               Image
%      x               State
%      par             Parameters
%      patches         Exsiting patches
%
%   Output
%      yfm             Measured feature position in metric scale
%      yfp             Measured feature position in pixels
%      yhatfp          Predicted feature position in pixels
%      yhatfp_noinit   Predicted feature positions that should not be initialized again
%      visible         Index to the map entries that are seen in the current image
%      visInnovation   Vision measurement innovation (y - yhat)

Tsize = par.search_region;  % Size of the seach region around each feature
Psize = par.Psize;          % Patch size, the size of the patches are Psize*Psize pixels
min_sep = par.min_sep;      % Minimum separation between map entries (features)
visible = [];               % Index to visible features
yhatfm_vis = [];            % Measured feature position metric, visible in the map
yfp = [];                   % Measured feature position in pixels
yfm = [];                   % Measured feature position metric
yhatfp=[];                  % Predicted feature position in pixels
yhatfp_noinit=[];           % Predicted feature positions that should not be initialized again
[maxy,maxx]=size(I);
for i = 1:(length(x)-par.nps)/3   % Loop over all map entries
  yhatfm = yhatf(1+(i-1)*2:i*2);  % Predicted feature position in metric scale
  yhatfp(1,i) = round(par.cam.fc(1).*yhatfm(1)+par.cam.cc(1)); % Predicted feature position in pixels (x-direction)
  yhatfp(2,i) = round(par.cam.fc(2).*yhatfm(2)+par.cam.cc(2)); % Predicted feature position in pixels (y-direction)
  
  % Is the current features within the FOV? (OBS x (column) and y (row))
  if (yhatfp(1,i)-Tsize>0 && yhatfp(1,i)+Tsize<maxx && yhatfp(2,i)-Tsize>0 && yhatfp(2,i)+Tsize<maxy)
    visible(end+1) = i;        % Index to the map entries that are seen in the current image
    patch = patches.I(:,:,i);
    yhatfm_vis(end+1:end+2,1) = yhatfm;
    I1 = I(yhatfp(2,i)-Tsize:yhatfp(2,i)+Tsize,yhatfp(1,i)-Tsize:yhatfp(1,i)+Tsize);   % Extract a search region around the feature
    NCC = normxcorr2(patch,I1);       % Calculate the normalized cross-correlation
    NCC=NCC(Psize-1:end-Psize,Psize-1:end-Psize);
    [pos.dom, ind] = maxmat(NCC);
    offset=Tsize-floor(Psize/2);
    yfp(:,end+1)=[yhatfp(1,i)+(ind(2)-offset); yhatfp(2,i)+(ind(1)-offset)]; % Measured feature position in pixels
    % Measured feature position in metric scale
    yfm(:,end+1) = normalize(yfp(:,end),par.cam.fc,par.cam.cc,par.cam.kc,par.cam.alpha_c)';
  end
  % Make sure that the same feature is not initialized again
  if ((yhatfp(1,i)>-min_sep && yhatfp(1,i)<maxx+min_sep && yhatfp(2,i)>-min_sep && yhatfp(2,i)<maxy+min_sep) && status.findNewLandmarks)
    yhatfp_noinit(:,end+1)=yhatfp(:,i);
  end
end
if ~isempty(yfm)
  visInnovation = yfm(:)-yhatfm_vis;
else
  visInnovation = [];
end;
