% This script runs the sensor fusion

clear;
addpath('ComputerVision')     % Access computer vision functions
addpath('Transformations')    % Access coordinate transformations
load UAVdata.mat;             % Load UAV data
%load helidata_hp-2_20.mat;

%=================================
%== Image processing parameters ==
%=================================
par.Harris_k = 0.03;       % k used in the Harris' corner criterion
par.sigma_smooth = 0.5;    % Standard deviation for the smoothing before differentiation
par.sigma_diff = 1;        % Standard deviation for the differentiation operator
par.Harris_th = 0.5e7;     % Lower threshold for the Harris criterion
par.max_patch = 10;        % Maximum number of patches in one image
par.search_region = 25;    % Size of the seach region around each feature (will be adaptive in the future)
par.min_sep = 40;          % Minimum separation between map entries (features)
par.Psize = 11;            % Patch size, the size of the patches are Psize*Psize pixels

%====================
%===   Geometry   ===
%====================
ge_mag = 9.82;
par.ge = ge_mag*[0 0 -1]';
par.cam.fc = cam.fc;   par.cam.cc = cam.cc;
par.cam.kc = cam.kc;   par.cam.alpha_c = cam.alpha_c;
par.rc = geometry.rc;      % Translation between camera and IMU

par.T = meas.Timu;         % Not the best solution, but OK....
par.nps = 10;              % dimension of the vehicle state vector

%======================
%===   Initialize   ===
%======================
% Initial covariances
Pp0=(0.05)^2*eye(3);       % Initial uncertainty in position
Pv0=(0.1)^2*eye(3);        % Initial uncertainty in position
Pq0=(0.00001)^2*eye(4);    % Initial uncertainty in quaternion
par.P0=blkdiag(Pp0,Pv0,Pq0);

% Initial states
par.q_bc=[0 1/sqrt(2) -1/sqrt(2) 0]';
par.q_be = R2q(geometry.Rbe);  % Rotation from earth to IMU (body)
p0=[0; 0; 0];                  % Initial position
v0=[0; 0; 0];                  % Initial velocity
psi=pi/2+atan(178/17);         % psi=atan(56/184)+pi/2;
T1=[cos(psi),sin(psi),0;
    -sin(psi),cos(psi),0;
    0,0,1];
q0=R2q(T1);

par.x0=[p0; v0; q0];

% Map states
par.Pf0 = diag([2^2 2^2 4^2]);  % Initial feature position uncertainty  m
par.zf0 = 58;                   % Initialize features at this height (in camera system)

% Covariances
par.Qa = par.T*20^2*ones(3,1);  % Ra, note the high noise level
par.Qw = 1e-6*ones(4,1);        % par.T*0.1^2*ones(4,1); % Rw
par.Rc=(0.001)^2;               % camera measurement noise (same in both directions)
par.Rbar = 10;                  % Barometer measurement noise

par.plotOnOff = 1;              % Switch for plots (1 - plots, 0 - no plots) Not that it is quite costly to produce plots
%====================
%===  Estimation  ===
%====================
x{1} = par.x0;          % Initialize the mean value of the state
P{1} = par.P0;          % Initialize the state covariance
status.ctime=1+1;       % Time index for the camera images

% Extract initial patches
im = meas.yim{1};
[x{1}, P{1}, patches] = extractLandmarksInit(im, x{1}, P{1}, par);

% Main loop
for t=2:300    %size(meas.ya,2)
  status.findNewLandmarks = (rem(t, 8) == 0); %Find new landmarks every 8th time
  status.t = t;
  [x{t},P{t},patches,visible,status] = SFTimeStep(x{t-1},P{t-1},meas,patches,par,status);
end;
res_vis.x = x;
res_vis.P = P;

if par.plotOnOff
  plotResults(ref, res_vis);
end;